# -*- coding: utf8 -*-
'''
Created on 10.12.2010

@author: gsu
'''

from account_logic import AccountingLogicService, ttypes
from plugins_base import BasePlugin, Queue, ThriftBinaryClient
from decimal import Decimal
from contextlib import closing

from logging import getLogger
logger = getLogger('su.artix.aqspy')

@Queue(name='artix.accounting')
class AccountingPlugin(BasePlugin):

    client = ThriftBinaryClient(AccountingLogicService.Client)

    def __init__(self):
        super(AccountingPlugin, self).__init__()

    def _tryHosts(self, hosts):
        for addr, port in hosts:
            logger.info(u'Пробуем %s:%s' % (addr, port))
            try:
                self.socket.host = addr
                self.socket.port = int(port)
                with closing(self.socket):
                    self.socket.open()
                    yield self.client
            except Exception, e:
                logger.error(u'Ой: %s' % e)
                continue
            
    def getAdditionalAssetInfo(self, message):
        assetInfo = None
        if u'additionalInfo' in message.content:
            assetInfo = AccountingLogicService.AdditionalAssetInfo(
                dateFromCash = long(message.content['additionalInfo']['date_cash'])*1000,
                cashCode = message.content['additionalInfo']['cashCode'],
                shiftNumber = message.content['additionalInfo']['shiftNumber'],
                checkNumber = message.content['additionalInfo']['checkNumber']
                )
        return assetInfo

    def handleMessage(self, message):
        logger.info(u'Получено сообщение: %s' % message)

        ok = False
        hosts = map(lambda x: x.split(':'), message.content['hosts'])
        for client in self._tryHosts(hosts):
            if message.content['method'] == 'add':
                func = client.add
                logger.info(u'Будем зачислять бонусы')
            elif message.content['method'] == 'sub':
                func = client.sub
                logger.info(u'Будем списывать бонусы')
            elif message.content['method'] == 'back':
                func = client.back
                logger.info(u'Будем отменять списание бонусов')
            elif message.content['method'] == 'commit':
                func = client.commitTransaction
                logger.info(u'Закрываем транзакцию подтверждением операции')
            elif message.content['method'] == 'rollback':
                func = client.rollbackTransaction
                logger.info(u'Закрываем транзакцию отменой операции')
            elif message.content['method'] == 'setAdditionalCardStatus':
                func = client.setAdditionalCardStatus
                logger.info(u'Устанавливаем дополнительный статус карты')
            elif message.content['method'] == 'revertEarnBySessionId':
                func = client.revertEarnBySessionId
                logger.info(u'Отменяем начисленные бонусы')
            else:
                func = None
            if func:
                args = []
                args.append(message.content['terminalId'])

                if message.content['method'] == 'setAdditionalCardStatus':
                    card = AccountingLogicService.Card(cardNumber = str(message.content['card']['cardNumber']),
                                                       accountNumber = str(message.content['card']['accountNumber']))

                    args.append(card)
                    args.append(message.content['additionalStatus'])
                elif message.content['method'] == 'rollback':
                    args.append(message.content['sessionId'])
                    args.append(message.content['cardNumber'])
                elif message.content['method'] == 'revertEarnBySessionId':
                    args.append(message.content['cardNumber'])
                    args.append(message.content['sessionId'])
                    args.append(message.content['parentSessionId'])
                    args.append(Decimal(message.content['amount']) * Decimal(100))
                    args.append(self.getAdditionalAssetInfo(message))
                elif message.content['method'] != 'commit':
                    args.append(message.content['sessionId'])
                    args.append(message.content['cardNumber'])
                    args.append(Decimal(message.content['amount']) * Decimal(100))
                    args.append(self.getAdditionalAssetInfo(message))

                    assetGroup = None
                    if 'assetGroup' in message.content:
                        assetGroup = AccountingLogicService.AssetGroup(
                                                           groupId = str(message.content['assetGroup']['groupId']),
                                                           beginDate = long(message.content['assetGroup']['data_beg'])*1000,
                                                           endDate = long(message.content['assetGroup']['data_end'])*1000,
                                                           weight = int(message.content['assetGroup']['weight'])
                                                           )
                    args.append(assetGroup)
                    logger.info(u"терминал: %s, сессия: %s, карта: %s, сумма (коп): %d" % (args[0],args[1],args[2],args[3]))
                else:
                    args.append(message.content['sessionId'])
                    args.append(message.content['cardNumber'])
                    dictStatus = ttypes.CardStatus._NAMES_TO_VALUES

                    cardStatusStr = message.content[u'cardStatus'] if u'cardStatus' in message.content else None
                    cardStatus = dictStatus[cardStatusStr] if cardStatusStr in dictStatus else None

                    args.append(cardStatus)

                    additionalCardStatus = message.content['additionalCardStatus'] if 'additionalCardStatus' in message.content else None
                    args.append(additionalCardStatus)
                    
                    logger.info(u"терминал: %s, сессия: %s, карта: %s, новый статус карты: %s, новый дополнительный статус %s"
                                % (args[0], args[1], args[2], cardStatusStr, additionalCardStatus.decode("utf-8")))

                result = func(*args)
                logger.info(u'Метод выполнен! Результат: %s' % result)

                try:
                    logger.info(result.description.decode("utf-8"))
                except Exception, e:
                    pass

                ok = result.success
                break

        if not ok:
            raise Exception(u'Сообщение не обработано: %s' % message.content)
