# -*- coding: utf8 -*-
'''
Created on 13.11.2012

@author: kov
'''

from plugins_base import Queue, BasePlugin
from logging import getLogger
from datetime import datetime
import socket
queue = 'artix.orders.bad'
logger = getLogger('su.artix.aqspy')

@Queue(name=queue)
class OrdersBasePlugin(BasePlugin):
    
    queueNameBad = queue
    
    def __init__(self, prefixLog = queue.upper()):
        super(OrdersBasePlugin, self).__init__()
        self.failTimeout = 300
        self.prefixLog = prefixLog
        
        
    def handleMessage(self, message):
        logger.info(u'%s: Получено сообщение: %s' % (self.prefixLog, message))        
        host = message.content["host"]
        port = message.content["port"]
        cut = message.content["useCutter"]
        receipt = message.content["receipt"].decode('utf8')
        font = message.content["font"]
        timeDeathStr = str(message.content["timeDeath"])
        currentTime = datetime.now()        

        logger.info("%s: host: %s" % (self.prefixLog, host))
        logger.info("%s: port: %s" % (self.prefixLog, port))
        logger.info("%s: font: %s" % (self.prefixLog, font))
        logger.info("%s: useCutter: %s" % (self.prefixLog, cut))
        logger.info("%s: timeDeath: %s" % (self.prefixLog, timeDeathStr))
        logger.info("%s: current time: %s" % (self.prefixLog, currentTime))
        logger.info("%s: receipt: " % self.prefixLog)
        logger.info("%s" % receipt)        
        
        timeDeath = datetime.strptime(timeDeathStr, '%Y-%m-%d %H:%M:%S')
        if timeDeath > currentTime:        
            sock = socket.socket
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((host, port))
            sock.send('\x1D\x61\x00')           # init
            self.setFont(sock, font)            # set font
            self.testForStatus(sock)            # test        
            sock.send(receipt.encode('cp866'))  # send text on printer
            sock.send('\x0A')        
            if cut:
                sock.send('\x1B\x69')           # cut
            sock.close()
        else:
            logger.info("%s: Время вышло. Удаляем сообщение." % self.prefixLog)

    def testForStatus(self, socket):
        res = ''
        for a in '\x01''\x02''\x03''\x04':
            socket.send('\x10\x04' + a)    
            res += socket.recv(1000)
    
        if int(res[3].encode("hex"), 16) & 0x60:
            raise Exception(u'Нет бумаги')
        if int(res[1].encode("hex"), 16) & 0x04:
            raise Exception(u'Открыта крышка')
        if int(res[1].encode("hex"), 16) & 0x08:
            raise Exception(u'Протяжка бумаги вручную')
        if int(res[1].encode("hex"), 16) & 0x20:
            raise Exception(u'Нет бумаги')
        if int(res[1].encode("hex"), 16) & 0x40:
            if int(res[2].encode("hex"), 16) & 0x08:
                raise Exception(u'Ошибка отрезчика')
            if int(res[2].encode("hex"), 16) & 0x20:
                raise Exception(u'Неустранимая ошибка')
            raise Exception(u'Ошибка устройства')

    def setFont(self, socket, fontType):
    
        res = '\x00'
        
        if (fontType == 'Normal'):          # нормальный
            res = '\x00'
        elif (fontType == 'Enlarge'):       # увеличенный
            res = '\x30'
        elif (fontType == 'Compressed'):    # сжатый
            res = '\x09'
        elif (fontType == 'DoubleWidth'):   # двойной ширины
            res = '\x20'
        elif (fontType == 'Smaller'):       # уменьшеный
            res = '\x01'
#        elif (fontType == 'DoubleHeight'):  # двойной высоты                - not working
#            res = '\x10'
        elif (fontType == 'Underlined'):    # подчеркнутый
            res = '\x80'
            
        socket.send('\x1B\x21' + res)
