# -*- coding: utf8 -*-
'''
Created on 19.04.2011

@author: gsu
'''

from plugins_base import BasePlugin, Queue
from decimal import Decimal
import json
import os
import httplib2
import urllib

from logging import getLogger
logger = getLogger('su.artix.aqspy')

COMMIT = u'commit'
ROLLBACK = u'rollback'
UNDEFINED = u'Undefined'

DIRERRORPAYMENT = u'/linuxcash/cash/data/qpid/paygate/'

@Queue(name='artix.paygate')
class PaygatePlugin(BasePlugin):

    def __init__(self):
        self.failTimeout = 300
        self.timeout = 30
        super(PaygatePlugin, self).__init__()
        
    def handleMessage(self, message):
        
        logger.info(u'Получено сообщение: %s' % message)
        ok = False
        
        typePayment = UNDEFINED
        if u'typePayment' in message.content:
            typePayment = message.content[u'typePayment']
        else:
            self.saveToFileErrorPayment(message)
            return
        logger.info(u'Полученный метод: %s' %typePayment)
        try:
            if typePayment in [ROLLBACK, COMMIT]:
                result, content = self.sendRequest(message, typePayment)
                if result['status'] == '200':
                    content = json.loads(content)
                    if content['result']['status'] == 0:
                        logger.info(u'Метод выполнен!')
                        ok = True
                    else:
                        logger.error(u'Ошибка выполнения метода, код ошибки: %s, ошибка: %s' 
                                     % (content['result']['status'], 
                                        content['result']['comment']))
                else:
                    logger.error(u'Ошибка REST-запроса, код ошибки: %s' % result['status'])
            else:
                self.saveToFileErrorPayment(message)
                ok = True
        except Exception as e:
            logger.error(u'Ошибка: %s' % e)
        if not ok:
            raise Exception(u'Сообщение не обработано: %s' % message.content)

    def sendRequest(self, message, typePayment):
        """
        Выполнение запроса
        """
        if typePayment == ROLLBACK:
            logger.info(u'Выполняем отмену платежа')
            method = "PUT"
        else:
            logger.info(u'Подтверждаем платеж')
            method = "POST"
        conn = httplib2.Http(timeout = self.timeout, disable_ssl_certificate_validation=True)
        conn.add_credentials(str(message.content[u'user']),str(message.content[u'password']))
        url = str(message.content[u'url'])
        # Косяк в httplib2, если тело пустое не заполняется Content-Length
        return conn.request(url, method, headers={'Content-Length':'0'})

    def saveToFileErrorPayment(self, message):
        """
        Сохранение в файл сообщения с неизвестным типом.
        """
        logger.info(u'Неизвестный метод. Сообщение будет удалено из очереди, файл платежа будет сохранен.')

        if not os.path.exists(DIRERRORPAYMENT):
            os.makedirs(DIRERRORPAYMENT)

        fileName = u'%s%s.undefined' % (DIRERRORPAYMENT, message.content[u'paymentId'])
        logger.info(u'Сохранение файла "%s"' % fileName)
        with open(fileName, u'w') as f:
            f.write(json.dumps(message.content))
        logger.info(u'Файл успешно сохранен.')

