# -*- coding: utf8 -*-
'''
Created on 10.12.2010

@author: gsu
'''

from thrift.transport import TSocket, TTransport
from thrift.protocol import TBinaryProtocol
import qpid.messaging.exceptions
import functools, time
from contextlib import contextmanager, closing
import logging
from logging import getLogger
logger = getLogger('su.artix.aqspy')

class Queue(object):

    def __init__(self, name, persistent=True, reply=False):
        self.name = name
        self.persistent = persistent
        self.reply = reply

    def __call__(self, cls):
        cls.queue = self
        return cls

class ThriftBinaryClient(object):
    
    def __init__(self, client_type, transport=TTransport.TBufferedTransport):
        self.client_type = client_type
        self.transport = transport

    def inject(self, name, inst):
        inst.socket = TSocket.TSocket()
        inst.socket.setTimeout(180000)
        inst.transport = self.transport(inst.socket)
        inst.protocol = TBinaryProtocol.TBinaryProtocol(inst.transport)
        setattr(inst, name, self.client_type(inst.protocol))

def wrapmethod(func, premethod=None, postmethod=None):
    def w(function, *args, **kwds):
        if premethod:
            premethod(*args, **kwds)
        function(*args, **kwds)
        if postmethod:
            postmethod(*args, **kwds)
    return lambda *x,**k: w(func,*x,**k)

class BasePluginType(type):
    def __new__(cls, name, bases, dct):
        return type.__new__(cls, name, bases, dct)
    def __init__(cls, name, bases, dct):
        super(BasePluginType, cls).__init__(name, bases, dct)

def sendMessage(session, receiver, sender, message):
    try:
        logger.info(u'Ставим сообщение %s в очередь' % message)
        sender.send(message)
    except Exception, e:
        logger.error(u'Ой: %s' % e)
        logger.info(u'Сообщение возможно потеряно!')
        raise e

@contextmanager
def fetchGuarantedMessage(session, receiver, sender, failTimeout):
    logger.debug(u'Ждём сообщений из очереди %s...' % session.name)
    message = receiver.fetch(timeout=20.0)
    try:
        logger.info(u'Есть сообщение: %s из очереди %s' % (message, session.name))
        yield message
        logger.info(u'Подтверждаем сессию')
        session.acknowledge()
    except Exception, e:
        logger.error(u'Ой: %s' % e)
        logger.exception(repr(e))
        logger.info(u'Возвращаем сообщение в очередь %s' % session.name)
        sendMessage(session, receiver, sender, message)
        logger.info(u'Подтверждаем сессию')
        session.acknowledge()
        logger.info(u'Ждём %f секунд' % failTimeout)
        time.sleep(failTimeout)

@contextmanager
def fetchMessage(session, receiver, sender, failTimeout):
    logger.debug(u'Ждём сообщений из очереди %s...' % session.name)
    message = receiver.fetch(timeout=20.0)
    try:
        logger.info(u'Есть сообщение: %s из очереди %s' % (message, session.name))
        logger.info(u'Подтверждаем сессию')
        session.acknowledge()
        yield message
    except Exception, e:
        logger.error(u'Ой: %s' % e)
        raise e

class BasePlugin(object):

    exit = False    

    def __init__(self):
        
        injectors = []
        def inject(self, val, name):
            val.inject(name, self)
        for name, value in vars(self.__class__).iteritems():
            if hasattr(value, 'inject'):
                injectors.append(functools.partial(inject, val=value, name=name))
        for injector in injectors:
            injector(self)

    def handleMessage(self):
        pass
    
    def sendMessage(self, message):
        sendMessage(None, None, self.sender, message)

    def _handleAndResponse(self, session, sender, message):
        try:
            responce = self.handleMessage(message)
        except ControlException, e:
            responce = str(e)
            raise e
        finally:
            try:
                if message.reply_to and self.queue.reply:
                    logging.error(u'отправка ответа %s по адресу %s' % (responce, message.reply_to))
                    with closing(sender):
                        sender = session.sender(str(message.reply_to))
                        sender.send(responce)
            except Exception, e:
                logging.error(u'ответ не отправлен: %s' % e)

    def run(self, connection):
        logger.info(u'Создаём сессию...')
        self.connection = connection
        session = connection.session()        
        with closing(session):
            logger.info(u'удачно')
            logger.info(u'Отправитель для %s...' % self.queue.name)
            self.sender = session.sender(self.queue.name)
            logger.info(u'создан')
            logger.info(u'Получатель для %s...' % self.queue.name)
            receiver = session.receiver(self.queue.name)
            logger.info(u'создан')

            fetch = fetchGuarantedMessage if self.queue.persistent else fetchMessage            
            failTimeout = self.failTimeout if hasattr(self, "failTimeout") else 60            

            while not self.exit:
                try:
                    with fetch(session, receiver, self.sender, failTimeout) as message:
                        self._handleAndResponse(session, self.sender, message)
                except qpid.messaging.exceptions.Empty:
                    logger.debug(u'Таймаут истёк')
                    session.acknowledge()

    def __str__(self):
        return self.__class__.__name__

class ControlException(Exception):

    def __init__(self, value):
        self.parameter = value

    def __str__(self):
        return repr(self.parameter)

    def __repr__(self):
        return repr(self.parameter)

class StopException(ControlException):

    def __init__(self, value):
        super(StopException, self).__init__(value)

class RestartException(ControlException):

    def __init__(self, value):
        super(RestartException, self).__init__(value)
